#!/usr/bin/perl
# 
# SubShifter v.0.4
# SubShift shifts timing of .sub-files
#
# Copyright (c) 2005 Joni Valtanen
# GNU General Public Licence v2
#
# How this works:
# -reads and parses command line arguments
# -reads given file line by line
# -parses start,stop and text from to line
# -increases or decreases subtitle start and stop value with given value
# -prints new shifted subtitles to file or stdout
#
#
# Command line arguments:
# -t shifting time in frames
# -i inputfile 
# -o outputfile (optional)
# -d minus decrease
# -s start time (optional)
# -e end time (optional)
#
#
# Exit codes:
# 0 - out from help and version
# 1 - not enought arguments
# 2 - no inputfile
# 3 - time is not numeric value
#
#
# Version information
#
# 0.1  -  051017, Joni Valtanen
#    * created
#
# 0.2  -  051017, Joni Valtanen
#    * added help(-h), version(-V) and verbode(-v) options
#    * changed function isTime to isNumerical
#      * changed name and changed exit to return
#    * cleaned usage function
#
# 0.3  -  051017, Joni Valtanen
#    * added from (-f) option and its functionality
#    * fixed usage
#
# 0.4 -  051027, Joni Valtanen
#    * added end (-e) option and its functionality
#    * changed -f option to -s
#    * fixed negative drop
#    * fixed usage
#
use Getopt::Std;

$version = "0.4";

# prints out usage information
sub usage
{
	print "\nSubShifter v$version\n\n";
	print "Usage: subshifter.pl -t <msec> -i <ifile>\n";
	print " Optional swiches:\n    [-s <frames>] [-e <frames>] [-o <ofile>] [-h] [-m] [-v] [-V]\n\n";
	print "  -t     frames to shift subtitles. Must be >0\n";
	print "  -i     filename to reading\n";
	print "  -s     start shifting from file after given frames\n";
	print "  -e     end shifting from file after given frames\n";
	print "  -o     filename to writing. If no name is given stdout is used.\n";
	print "  -d     used to decrease shiftingtime \(frames-frameshifting\)\n";
	print "  -h     prints this help\n";
	print "  -v     verbose mode\n";
	print "  -V     prints only version information\n\n";
}

# is numerical value
# decimal numbers are not allowed eighter
sub isNumerical
{
	($test) = @_;
	if (!($test !~ m/\D/) )
	{
		&usage();
    		return 0;
	}
	return 1;
}

# Get options
getopt('tiseo', \%opts);

$input = $opts{'i'};
$output = $opts{'o'};
$time = $opts{'t'};
$from = $opts{'s'};
$end = $opts{'e'};

if($opts{'V'})
{
	print "\nSubShifter v$version\n\n";
	exit 0;
}

# help
if($opts{'h'})
{
	&usage();
	exit 0;
}

# Some tests are arguments ok
if(!$time || !$input )
{
	&usage;
	exit 1;
}

if (!(-e $input))
{
	&usage();
	exit 2;
}

if(!&isNumerical($time))
{
	exit 3;
}

# decrease
if($opts{'d'})
{
	$time = -$time;
}

if(!$from)
{
	$from = 0;
}
if(!$end)
{
	$end = 999999;
}

#print some information
if($opts{'v'})
{
print "Time to shift: $time \n";
print "from $input\n";
print "to file $output\n\n";
}

open IFILE, "$input" or die("Couldn't open $input\n") ;

# if user gave output file
# set sdtout to print this
if($output)
{
	open STDOUT, ">$output" or die("Couldn't open $output\n") ;
}

# reads whole file line by line
# and writes it to stdout
while($test =  readline(IFILE))
{
	$line = $test;
	@line_t1 = split(/}/, $line);
	@line_t2 = split(/{/, $line_t1[0]);
	@line_t3 = split(/{/, $line_t1[1]);

	# from
	if( ( $from <= $line_t2[1] ) && ( $end >= $line_t3[1] ) )
	{
	  $line_t2[1] += $time;
	  $line_t3[1] += $time;
	}
	# drops negatives out ;)
	if( $line_t2[1] >= 0 )
	{
	  print "{$line_t2[1]}";
	  print "{$line_t3[1]}";
	  print "$line_t1[2]";
	}
}

#close everything
close IFILE;

if($output)
{
	close STDOUT;
}


